# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

from .files import ATRFiles, ATRFile
from .cli import main

__author__ = 'XaviTorello'
__version__ = '0.4.6'
